/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language1.*;

/**
   This program parses and then evaluates strings from Language_1.
*/
public class Language_1_Examples
{
   public static void main(String[] args)
   {
      String[] exprs = {
         "(+ 1 2 ) )",       // invalid expression
         "(- 1 2 3)",        // invalid expression
         "(* 3)",            // invalid expression
         "(! true false)",   // invalid expression
         "x",                // invalid expression
         "(@ 2 3)",          // invalid expression
         "(+ 2 false)",      // invalid expression
         "(! 2)",            // invalid expression
         "(&& 1 false)",     // invalid expression
         "(|| false 0)",     // invalid expression
         "(! (+ 1 1))",      // invalid expression

         "(+ 2 (* 5 5 2))",
         "(+ (* 5 5 2) 2)",
         "(- (* ( + 3 3 2) 4) 9)",
         "(- 9 (* 4 (+ 3 2 3)))",
         "(+ (/ (* 32 41) 2) (* 12 48))",
         "(! false)",
         "(|| true false)",
         "(! (&& true false))",
         "(< 3 5)",
         "(&& (< 3 5) (! (> 3 0)))",
         "(== (+ 4 2) (* 2 3))",
         "(== (> 4 2) (< 2 3))",
         "(+ 2 (* 5 5) (/ 4 2))",
         "(+ (- 10) (+ 10))",
         "(* (+ 2 3) (+ 4 5) (+ 5 7))",
         "(&& (! (> 1 2)) (== (- (- 2 4)) 2))",
         "(|| true (!= (/ 1 0) 0))",
         "(&& (== (- 1 1) 0) (< (- 1) 1) false (== 0 (/ 1 0)))",
         "(+ 1 (** 2 1) (** 2 2) (** 2 3) (** 2 4) (** 2 5) (** 2 6))"
      };

      int i = 0;
      for (final String str : exprs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the expression.
         try
         {
            System.out.print("Parsing exp "+i+": \"" + str + "\"" );
            final Tree<Token> ast = Parse2AST.parse( str );
            System.out.println( " ... Parsed" );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.print("\"Unparsed\" exp: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_1.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            System.out.print("Evaluating exp: " );

            // Evaluate the expression (interpret the AST).
            final Value result = EvaluateAST.eval( ast );
            System.out.println( "result = " + result + "\n" );
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
