/*

*/

package language1;

/**
   This class is a kind of "tagged union".
   <p>
   Each instance of this class holds a value for our simple language.
   But our language has two kinds of values, ints and booleans. So
   each instance of this class holds both an int value and a boolean
   value. The "tag" field is used to determine which of the two
   values that an instance holds, the int or the boolean, is valid.
   <p>
   See <a href="https://en.wikipedia.org/wiki/Tagged_union" target="_top">
                https://en.wikipedia.org/wiki/Tagged_union</a>
*/
public class Value
{
   public String tag;     // either "int" or "bool"
   public int valueI = 0;
   public boolean valueB = false;

   public static final String INT_TAG = "int";
   public static final String BOOL_TAG = "bool";

   /**
      Construct a {@code Value} object that holds an int value.

      @param value  the integer value for this {@code Value} object
   */
   public Value(int value)
   {
      this.tag = INT_TAG;
      this.valueI = value;
      // Implement C/C++ like semantics.
      this.valueB = (value != 0);
   }


   /**
      Construct a {@code Value} object that holds a boolean value.

      @param value  the boolean value for this {@code Value} object
   */
   public Value(boolean value)
   {
      this.tag = BOOL_TAG;
      this.valueB = value;
      // Implement C/C++ like semantics.
      this.valueI = (value ? 1 : 0);
   }


   @Override
   public String toString()
   {
      return "[tag->" + tag + ", valueI->" + valueI + ", valueB->" + valueB + "]";
   }


   /**
      @return a simplified {@link String} representation of this {@code Value}
   */
   public String toSimpleString()
   {
      String result = "";

      if ( tag.equals(BOOL_TAG) )
      {
         result += valueB;
      }
      else if ( tag.equals(INT_TAG) )
      {
         result += valueI;
      }
      else // bad tag (shouldn't get here)
      {
         result += "[tag->" + tag + ", valueI->" + valueI + ", valueB->" + valueB + "]";
      }

      return result;
   }
}
