/*

*/

package treelanguage;

import tree.*;
import tokenizer.*;

/**
   The prettyPrint() method takes a {@link Tree}
   of {@link Token}s and converts it into a well
   formatted {@link String}.
*/
public class PrettyPrintAST
{
   public static final String INDENTATION = "  ";

   /**
      Pretty print a {@link Tree} of {@link Token}s
      with each sub tree indented by a certain amount.

      @param tree  {@link Tree} of {@link Token}s to be pretty printed
      @return {@link String} representation of the prettyprinted {@link Tree}
   */
   public static String prettyPrint(final Tree<Token> tree)
   {
      return prettyPrint(tree, "");
   }


   /**
      This is a recursive helper function for the prettyPrint method.
      <p>
      This method pretty prints a {@link Tree} with a given amount
      of indentation. If the pretty printer needs to prettyprint a
      sub tree, then the sub tree gets a larger amount of indentation.
      <p>
      This method is essentially a pre-order traversal of the {@link Tree}.

      @param tree    {@link Tree} of {@link Token}s to be pretty printed
      @param indent  {@link String} holding the current left margin indentation
      @return {@link String} representation of the given {@link Tree} prettyprinted and indented
   */
   private static String prettyPrint(final Tree<Token> tree,
                                     final String indent)
   {
      String result = indent;

      if ( tree.isLeaf() ) // stops the recursion
      {
         result += tree.root().lexeme;
      }
      else
      {
         result += "(" + tree.root().lexeme;

         // recursively traverse all the sub trees
         for ( final Tree<Token> subTree : tree.subTrees() )
         {
            result += "\n" + prettyPrint(subTree, indent + INDENTATION);
         }

         result += "\n" + indent + ")";
      }

      return result;
   }
}
